#include <Wire.h>
#include "botCntrlBase.h"

RemoteControl remoteCntrl;
PreSetPath preSetPath;
SeekLight seekLight;
BotCntrlBase* currentCntrl = 0;

unsigned long lastModeButton;

/*********************************************************************************
 */
void setup(void)
{
  BotCntrlBase::init();

  pinMode(debugLEDJC3, OUTPUT);

  attachInterrupt(leftWheelSensor - 2, leftWheelNotify, FALLING);
  attachInterrupt(rightWheelSensor - 2, rightWheelNotify, FALLING);
    
  pinMode(modeSwitch, INPUT);
  lastModeButton = millis();

  pinMode(bluetoothPower, OUTPUT); 
  digitalWrite(bluetoothPower, HIGH);
//  digitalWrite(bluetoothPower, LOW);

  Serial.begin(115200);
//  Serial.begin(9600);
  Wire.begin();
  
  // give everything else time to start up
  delay(1000);
}

/*********************************************************************************
 */
void loop(void)
{
  if(currentCntrl == 0)
  {
    changeMode(CMD_SET_PATH);
  }

  unsigned long now = millis();
  if(now - lastModeButton > 250 && digitalRead(modeSwitch))
  {
    lastModeButton = now;
    switch(currentCntrl->getMode())
    {
    case CMD_REMOTE_CONTROL:
      changeMode(CMD_SET_PATH);
      break;
    case CMD_SET_PATH:
      changeMode(CMD_SEEK_LIGHT);
      break;
    case CMD_SEEK_LIGHT:
      changeMode(CMD_REMOTE_CONTROL);
      break;
    }
  }
  currentCntrl->doLoop();
}

/*********************************************************************************
 */
void leftWheelNotify()
{
  BotCntrlBase::ulLeftWheelCnt++;
}

/*********************************************************************************
 */
void rightWheelNotify()
{
  BotCntrlBase::ulRightWheelCnt++;
}

/*********************************************************************************
 */
void changeMode(char chMode)
{
  if(currentCntrl != 0)
    currentCntrl->stopping();
  
  switch(chMode)
  {
  case CMD_REMOTE_CONTROL:
    currentCntrl = &remoteCntrl;
    break;

  case CMD_SET_PATH:
    currentCntrl = &preSetPath;
    break;
    
  case CMD_SEEK_LIGHT:
    currentCntrl = &seekLight;
    break;
    
  }
  Wire.beginTransmission(I2C_DISPLAY);
  Wire.send(CMDS_MODE);
  Wire.send(currentCntrl->getMode());
  Wire.endTransmission();
  currentCntrl->restarting();
}

