/*********************************************************************************
 */
#ifndef MY_SHIFT_OUT_H
#define MY_SHIFT_OUT_H

#include <WProgram.h>
#include "botProtocol.h"
#include "Cmd.h"

//Standard PWM DC control 
const int M1 = 4;    //M1 Direction Control
const int E1 = 5;    //M1 Speed Control
const int E2 = 6;    //M2 Speed Control
const int M2 = 7;    //M1 Direction Control

// wheel sensors must be on pin 2 and 3 as they are attached to the interupts
const int leftWheelSensor  = 3;
const int rightWheelSensor = 2;

const int leftProxSensor   = 11;
const int rightProxSensor  = 10;

// 0  rx  (bluetooth)
// 1  tx  (bluetooth)

const int debugLEDJC3 = 9;

const int bluetoothPower = 12;
const int modeSwitch = 8;

// Available digital pins
// 13

const int batSensor = A0;
const int ampSensor = A1;
const int leftBackLightSensor = A6;
const int rightBackLightSensor = A7;
const int leftFrontLightSensor = A2;
const int rightFrontLightSensor = A3;

// A5 used for i2c comms
// A4 used for i2c comms

// Available analogue pins
// none


/*********************************************************************************
 */
class BotCntrlBase
{
public:
  static void init();
  virtual void restarting();
  virtual void stopping() = 0;
  virtual char getMode() = 0;
  void doLoop();
  
  // never reset these (you may get threading type issues)
  static volatile unsigned long ulRightWheelCnt;
  static volatile unsigned long ulLeftWheelCnt;
  
protected:
  void setCourseLeft(int iLeftSpeed);
  int getCourseLeft();
  void setCourseRight(int iRightSpeed);
  int getCourseRight();
  void displayLight(Cmd cmd);

  virtual void processMsg(CmdVal cmdVal);
  virtual void processMsg(Cmd cmd);
  virtual void readSensors() = 0;
  
private:
  int getCmdVal();
  void checkMsg();
  void processMsg();
  void driveMotors();
  void sendSensToDisplay();

  // vars for messages recieved
  static char szInBuf[32];
  static int iBufUsed;
  static boolean bMsgWaiting;

  // vars for direction
  int iLeftVal;
  int iRightVal;

  boolean bChangeSpeed;
  
  static unsigned int ulLastDispUpd;
};


/*********************************************************************************
 */
class RemoteControl : public BotCntrlBase
{
public:
  virtual void restarting();
  virtual void stopping();
  virtual char getMode() { return CMD_REMOTE_CONTROL; }

protected:
  virtual void processMsg(CmdVal cmdVal);
  virtual void processMsg(Cmd cmd);
  virtual void readSensors();

private:
  unsigned long ulLastMsg;
  int lastLeftProx;
  int lastRightProx;
  char currDisp;
};


/*********************************************************************************
 */
class PreSetPath : public BotCntrlBase
{
public:
  virtual void restarting();
  virtual void stopping();
  virtual char getMode() { return CMD_SET_PATH; }

protected:
  virtual void processMsg(Cmd cmd);
  virtual void readSensors();

private:
  unsigned long ulLeftStart;
  unsigned long ulRightStart;
  char chDoing;
};


/*********************************************************************************
 */
class SeekLight : public BotCntrlBase
{
public:
  virtual void restarting();
  virtual void stopping();
  virtual char getMode() { return CMD_SEEK_LIGHT; }

protected:
  virtual void readSensors();

private:
  unsigned long ulLeftStart;
  unsigned long ulRightStart;
  int iDoing;
};

/*********************************************************************************
 */
#endif //MY_SHIFT_OUT_H
